# Author:: Gerhard Spitzlsperger
# EMAIL and WWW:: ruby@gs68.de, www.gs68.de
# Copyright:: Copyright (c) 2003 Gerhard Spitzlsperger. All rights reserved.
# License:: LGPL see file LICENSE.
# Version 0.1

require 'fox'
require 'GS_PlotCanvas'

include Fox

# A little drawing canvas to use in our tests for float coordinates
class GS_FramePlotCanvas < GS_PlotCanvas
  attr_reader     :xaxis, :yaxis, :top_axis, :right_axis
  attr_reader     :titleStyle
  attr_accessor   :title

  def initialize(p, tgt, sel, opts, area, axis)
    # Invoke base class initialize function first
    super(p, tgt, sel, opts, area)  
    
    # margins on the canvas side 
    @left_margin = 50
    @right_margin = 50
    @top_margin = 50
    @bottom_margin = 50
    
    @title = ""
    @titleStyle = GS_Style.new
    @titleStyle.fontSize = 18

    # define axis and frame, the edges are implemented as axis to show 
    # right scaling behaviour 
    # prepare for exceptions
    begin 
      # check which x axis to plot
      if axis =~ /^lin.../
	@xaxis = GS_Axis.new("GS_XAxis_Autoscale", 
	                      @x_min, @y_min, @x_max, @y_min, 1.0, 1.0, true)
	@top_axis = GS_Axis.new("GS_XAxis_Autoscale", 
	                      @x_min, @y_max, @x_max, @y_max, 1.0, 1.0, true)	
        @x_scale = "lin"			      
      elsif axis =~ /^log.../		      
        @xaxis = GS_LogAxis.new("GS_LogXAxis_Autoscale", 
	                     @x_min, @y_min, @x_max, @y_min)
	@top_axis = GS_LogAxis.new("GS_LogXAxis_Autoscale", 
	                             @x_min, @y_max, @x_max, @y_max)	
	@x_scale = "log"			     
      else
        Kernel::raise UnexpectedAxisConstantException, 
	  "unexpected axis constant"
      end
      # check which y axis to plot
      if axis =~ /...lin$/	
	@yaxis = GS_Axis.new("GS_YAxis_Autoscale", 
	                      @x_min, @y_min, @x_min, @y_max, 1.0, 1.0, true)
	@right_axis = GS_Axis.new("GS_YAxis_Autoscale", 
	                      @x_max, @y_min, @x_max, @y_max, 1.0, 1.0, true)	
	@y_scale = "lin"			      
      elsif axis =~ /...log$/
        @yaxis = GS_LogAxis.new("GS_LogYAxis_Autoscale", 
	                        @x_min, @y_min, @x_min, @y_max) 
        @right_axis =  GS_LogAxis.new("GS_LogYAxis_Autoscale", 
	                        @x_max, @y_min, @x_max, @y_max) 
	@y_scale = "log"			
      else
        Kernel::raise UnexpectedAxisConstantException, 
	  "unexpected axis constant"
      end
    end # exception handling        

    @top_axis.ticks_and_numbers = false 
    @right_axis.ticks_and_numbers = false	
  end
  
  # plots everything on the canvas
  def draw
    dc = GS_FramePlotContext.new(self)
    dc.plot
  end  
end

class GS_FramePlotContext < GS_PlotContext
  def initialize(pcanvas)
    # Invoke base class initialize function first
    super(pcanvas)
  end
  
  def plot()
    # call the plot function of the base class
    super
    
    # Set the foreground color for drawing
    self.foreground = FXColor::Black 
  
    # first draw the axis and the frame, to account for possibly
    # changed coordinates resetting is done
    @pc.xaxis.setX(@pc.x_min, @pc.x_max, @pc.y_min)
    plotAxis(@pc.xaxis)
    
    @pc.yaxis.setY(@pc.y_min, @pc.y_max, @pc.x_min )
    plotAxis(@pc.yaxis)

    @pc.top_axis.setX(@pc.x_min, @pc.x_max, @pc.y_max )
    plotAxis(@pc.top_axis)
    
    @pc.right_axis.setY(@pc.y_min, @pc.y_max, @pc.x_max )
    plotAxis(@pc.right_axis)
    
    # plot the title in the upper margin
    @pc.titleStyle.adjustDC(self)
    drawText(@pc.left_margin+10, @pc.top_margin*0.7, @pc.title)
  end
  
  
end
  
# define exceptions
class UnexpectedAxisConstantException < RuntimeError
  attr :okToRetry
  def initialize(okToRetry)
    @okToRetry = okToRetry
  end
end
